/*
    include/comedi.h (installed as /usr/include/comedi.h)
    header file for comedi

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _COMEDI_H
#define _COMEDI_H

#ifdef __cplusplus
extern "C" {
#endif

/* versioning information */
#define COMEDI_VERSION "0.6.0"
#define COMEDI_VERSION_CODE (0*0x1000+6*0x100+0)

/* comedi's major device number */
#define COMEDI_MAJOR 98

/*
   maximum number of minor devices.  This can be increased, although
   kernel structures are currently statically allocated, thus you
   don't want this to be much more than you actually use.
 */
#define COMEDI_NDEVICES 4

/* number of config options in the config structure */
#define COMEDI_NDEVCONFOPTS 32

/* max length of device and driver names */
#define COMEDI_NAMELEN 20


typedef unsigned int sampl_t;

/* packs and unpacks a channel/range number */

#define CR_CHRNG(chan,rng)	((((rng)&0xffff)<<16) | ((chan)&0xffff))

#define CR_CHAN(a)	((a)&0xffff)
#define CR_RANGE(a)	(((a)>>16)&0xffff)

/* trigger flags */

#define TR_BOGUS	0x8000		/* do the motions */

/* channel flags */

#define CF_DITHER	0x0100		/* enable dithering */
#define CF_DEGLITCH	0x0200		/* enable deglitching */

#define CF_AREF_MASK	0x0003
#define CF_GROUND	0x0001		/* analog ref = analog ground */
#define CF_COMMON	0x0002		/* analog ref = analog common */
#define CF_DIFFERENTIAL	0x0003		/* analog ref = differential */

/* subdevice flags */

#define SDF_BUSY	0x0001		/* device is busy */
#define SDF_BUSY_OWNER	0x0002		/* device is busy with your job */
#define SDF_LOCKED	0x0004		/* subdevice is locked */
#define SDF_LOCK_OWNER	0x0008		/* you own lock */
#define SDF_MAXDATA	0x0010		/* maxdata depends on channel */
#define SDF_FLAGS	0x0020		/* flags depend on channel */
#define SDF_RANGETYPE	0x0040		/* range type depends on channel */

/* ioctls */

#define CIO 'd'
#define COMEDI_DEVCONFIG _IOW(CIO,0,comedi_devconfig)
#define COMEDI_DEVINFO _IOR(CIO,1,comedi_devinfo)
#define COMEDI_SUBDINFO _IOR(CIO,2,comedi_subdinfo)
#define COMEDI_CHANINFO _IOR(CIO,3,comedi_chaninfo)
#define COMEDI_TRIG _IOWR(CIO,4,comedi_trig)
#define COMEDI_LOCK _IO(CIO,5)
#define COMEDI_UNLOCK _IO(CIO,6)



/* structures */

typedef struct comedi_trig_struct comedi_trig;
typedef struct comedi_chaninfo_struct comedi_chaninfo;
typedef struct comedi_subdinfo_struct comedi_subdinfo;
typedef struct comedi_devinfo_struct comedi_devinfo;
typedef struct comedi_devconfig_struct comedi_devconfig;

struct comedi_trig_struct{
	unsigned int subdev;		/* subdevice */
	unsigned int mode;		/* mode */
	unsigned int flags;
	unsigned int n_chan;		/* number of channels */
	unsigned int *chanlist; 	/* channel/range list */
	sampl_t *data;			/* data list */
	unsigned int n;			/* number of scans */
	unsigned int trigsrc;
	unsigned int trigvar;
	unsigned int trigvar1;
};

struct comedi_chaninfo_struct{
	unsigned int subdev;
	sampl_t	*maxdata_list;
	unsigned int *flaglist;
	unsigned int *rangelist;
};


struct comedi_subdinfo_struct{
	unsigned int n_chan;
	unsigned int subd_flags;
	unsigned int timer_type;
	unsigned int len_chanlist;
	sampl_t		maxdata;
	unsigned int	flags;		/* channel flags */
	unsigned int	range_type;
};

struct comedi_devinfo_struct{
	unsigned int version_code;
	unsigned int n_subdevs;
	char driver_name[COMEDI_NAMELEN];
	char board_name[COMEDI_NAMELEN];
	int options[COMEDI_NDEVCONFOPTS];
};

struct comedi_devconfig_struct{
	char board_name[COMEDI_NAMELEN];
	int options[COMEDI_NDEVCONFOPTS];
};


#if 0
typedef struct{
	unsigned int chan;
	int io;			/* change this to config or something */
	unsigned int n;		/* change this to n_samples */
	int trigsrc;
	int trigvar;
	sampl_t data;
	unsigned int job;
}comedi_trig;

typedef struct{
	int io;
	unsigned int n1;	/* number of channels in scan */
	int trigsrc1;		/* time between samples */
	int trigvar1;
	unsigned int *chanlist;
	unsigned int n2;	/* number of scans */
	int trigsrc2;		/* time between scans */
	int trigvar2;
	unsigned int job;
}comedi_mtrig;

typedef struct{
	unsigned int job;
	unsigned int chan;
	sampl_t data;
}comedi_sample;


typedef struct{
	unsigned int chan;
	unsigned int flags;
	unsigned int pnum;
	int pval;
}comedi_param;

typedef struct{
	char driver_name[COMEDI_NAMELEN];
	char board_name[COMEDI_NAMELEN];
	int chan;
	int iobase;
	int iosize;
	int irq;
	int minor;
	unsigned int version_code;
}comedi_info;

typedef struct{
	void *local;
	unsigned int remote;
	unsigned int nbytes;
}comedi_memcpy;
#endif

/* ioctls */

#if 0

#define CIO 'd'
#define COMEDI_CONFIG _IOW(CIO,0,comedi_devconfig)
#define COMEDI_GETINFO _IOR(CIO,1,comedi_info)
#define COMEDI_TRIG _IOWR(CIO,2,comedi_trig)
#define COMEDI_MTRIG _IOWR(CIO,3,comedi_mtrig)

#define COMEDI_GETPARAM _IOWR(CIO,17,comedi_param)
#define COMEDI_SETPARAM _IOW(CIO,18,comedi_param)
#define COMEDI_GETNPARAM _IOWR(CIO,19,comedi_param)

#define COMEDI_COPY_FROM _IOW(CIO,32,comedi_memcpy)
#define COMEDI_COPY_TO _IOW(CIO,33,comedi_memcpy)
#define COMEDI_MEMSET _IOW(CIO,34,comedi_memcpy)
#define COMEDI_RESET _IO(CIO,35)
#define COMEDI_EXEC _IO(CIO,36)
#define COMEDI_LOCK _IO(CIO,37)
#define COMEDI_UNLOCK _IO(CIO,38)

#endif

#ifdef __cplusplus
}
#endif

#endif /* _COMEDI_H */

